""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 11/02/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetApPhot

Remarks :

History : (11/02/2016) First version.
"""



import sep
import numpy
import SRPGW as GW


def SexPhot(x,y,data,rds=(5,10,15),backgr=True):
    dat = data.astype(numpy.float32)
    if backgr:
        flux,fluxerr,flag = sep.sum_circle(dat,x,y,rds[0],bkgann=(rds[1],rds[2]),gain=GW.VSTgain)
    else:
        flux,fluxerr,flag = sep.sum_circle(dat,x,y,rds[0],gain=GW.VSTgain)
    #
    return flux, fluxerr



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 07/12/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)

Remarks :

History : (21/01/2016) First version.
"""


import numpy
from astropy.io.fits import getdata
from astropy.wcs import WCS
from astropy.stats import sigma_clipped_stats



def GetTest(rad,decd,fname,size=5):
    dat = getdata(fname)
    w = WCS(fname)
    #
    x, y = w.all_world2pix(rad,decd,0)
    #
    a = numpy.rint(x)-1
    b = numpy.rint(y)-1
    aa = a.astype(numpy.int)
    bb = b.astype(numpy.int)
    #
    res = []
    for i,l in zip(aa,bb):
        datf = dat[l-size:l+size,i-size:i+size]
        mean, median, std = sigma_clipped_stats(datf,iters=5)
        #mean = numpy.mean(datf)
        #std =  numpy.std(datf)
        res.append(mean)
        #
    return res



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.1.0
Author  : Stefano Covino
Date    : 25/03/2019
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   :

Remarks :

History : (23/12/2016) First version.
        : (25/03/2019) Fluxes rather than magnitudes.

"""

import numpy as np
from SRP.SRPStatistics.WeightedMean import WeightedMean
import SRPGW as GW


def Chi2(tab):
    res = []
    #
    fs = []
    efs = []
    #
    for e in tab.columns.keys():
        if GW.FLUX in e and e.find(GW.FLUX) == 0:
            fs.append(e)
    if len(fs) == 0:
        return -99
    #
    for e in tab.columns.keys():
        if GW.eFLUX in e:
            efs.append(e)
    if (len(efs) == 0) or (len(fs) != len(efs)):
        return -99
    #
    for e in tab:
        c = [(e[i],e[j]) for i,j in zip(fs,efs)]
        res.append(WeightedMean(c)[3])
    return res



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.2.2
Author  : Stefano Covino
Date    : 14/03/2019
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetPSFPhot

Remarks :

History : (25/02/2016) First version.
        : (26/02/2016) Minor improvement.
        : (13/12/2016) ApyPSF photometry added.
        : (16/12/2016) FWHM for PSF computation.
        : (14/03/2019) Change in output.
"""


from astropy.io.fits import getdata
import SRPGW as GW
from SRPGW.Flux2Mag import Flux2Mag
from SRPGW.DaoPSFPhot import DaoPSFPhot
from SRPGW.ApyPSFPhot import ApyPSFPhot



def GetPSFPhot(x,y,px,py,fname,rds=(5,10,15),fwhm=5,zp=30.0,sept='dao',cent=False,bck=True):
    data = getdata(fname)
    #
    if sept=='dao':
        flux,fluxerr,pflux,pfluxerr = DaoPSFPhot(x,y,px,py,data,rds,fwhm,zp,cent,bck)
        resa = Flux2Mag(flux.flatten(),fluxerr.flatten(),zp)
        resp = Flux2Mag(pflux,pfluxerr,zp)
    elif sept=='apy':
        aflux,afluxerr,pflux,pfluxerr = ApyPSFPhot(x,y,px,py,data,rds,fwhm,zp,cent)
        resa = Flux2Mag(aflux,afluxerr,zp)
        resp = Flux2Mag(pflux,pfluxerr,zp)
    #
    return resa['mags'], resa['emags'], resp['mags'], resp['emags'], resp['fluxes'], resp['efluxes']
    



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.4.6
Author  : Stefano Covino
Date    : 14/03/2019
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetApPhot

Remarks :

History : (17/11/2015) First version.
        : (12/12/2015) Better management of the upper limits.
        : (18/12/2015) Again a minor correction for the upper limits.
        : (28/01/2016) Possible centering before photometry.
        : (02/02/2016) nan handling.
        : (09/02/2016) Background subtraction choosable.
        : (25/02/2016) DaoPhot-like aperture photometry.
        : (26/02/2016) Report new positions if requested.
        : (27/07/2016) Syntax error correction.
        : (14/12/2016) Bug correction.
        : (19/04/2017) SRPFITS added.
        : (14/03/2019) Change in output.
"""


from astropy.io.fits import getdata
import SRPGW as GW
import numpy
from SRPFITS.Photometry.centerMoment import centerMoment
from SRPGW.SexPhot import SexPhot
from SRPGW.MyPhot import MyPhot
from SRPGW.ApyPhot import ApyPhot
from SRPGW.DaoPhot import DaoPhot
from SRPGW.Flux2Mag import Flux2Mag



def GetApPhot(x,y,fname,rds=(5,10,15),zp=30.0,sept='my',cent=False,bck=True):
    data = getdata(fname)
    #
    if cent:
        xx = []
        yy = []
        for i,l in zip(x,y):
            a,b = centerMoment(data,i,l,rds[1])
            xx.append(a)
            yy.append(b)
    else:
        xx = x
        yy = y
    #
    if sept == 'sex':
        flux,fluxerr = SexPhot(xx,yy,data,rds,backgr=bck)
    elif sept == 'my':
        flux,fluxerr = MyPhot(xx,yy,data,rds,backgr=bck)
    elif sept == 'apy':
        flux,fluxerr = ApyPhot(xx,yy,data,rds,backgr=bck)
    elif sept == 'dao':
        flux,fluxerr = DaoPhot(xx,yy,data,rds,backgr=bck)
    #
    res = Flux2Mag(flux,fluxerr,zp)
    #
    return res['mags'],res['emags'],res['fluxes'],res['efluxes']
    



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.2
Author  : Stefano Covino
Date    : 19/04/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetApPhot

Remarks :

History : (11/02/2016) First version.
        : (13/02/2016) Improved error estimate.
        : (19/04/2017) SRPFITS added.
"""



import SRP.SRPAstro as SRPAstro
import SRPGW as GW
import numpy
from SRPFITS.Photometry.getBackground import getBackground


def MyPhot(x,y,data,rds=(5,10,15),backgr=True):
    flux = []
    fluxerr = []
    for i,l in zip(x,y):
       totf,npix,maxf = SRPAstro.sumApert(data,i-1,l-1,rds[0])
       if backgr:
            bg,sbg,ebg,chbg,nbg = getBackground(data,i-1,l-1,rds[1],rds[2])
       else:
            bg = 0.0
            ebg = 0.0
       #
       tflux = totf-bg*npix
       tfluxerr = numpy.sqrt(numpy.fabs(totf)*GW.VSTgain + npix*(ebg*GW.VSTgain)**2 + npix*GW.VSTron)
       flux.append(tflux)
       fluxerr.append(tfluxerr)
    #
    return numpy.array(flux),numpy.array(fluxerr)
    



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.1.0
Author  : Stefano Covino
Date    : 10/02/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetHeadVal

Remarks :

History : (06/11/2015) First version.
        : (10/02/2916) Simpler and better version.
"""


from astropy.io.fits import getval



def GetHeadVal(headentry, fname):
    try:
        hdv = getval(fname, headentry)
    except KeyError:
        hdv = -99
    #
    return hdv


""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.1.0
Author  : Stefano Covino
Date    : 14/03/2019
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : 

Remarks :

History : (25/02/2016) First version.
        : (19/04/2017) SRPFITS added.
        : (21/12/2017) Minor improvement.
        : (14/03/2019) Fluxes in output
"""

import numpy
from SRPFITS.Photometry.Counts2Mag import Counts2Mag



def Flux2Mag(flux,fluxerr,zp=25):
    mag = []
    emag = []
    oflux = []
    oeflux = []
    for f, ef in zip(flux,fluxerr):
        if f > 0 and ef > 0:
            m,em = Counts2Mag(f,ef,zp)
            of,oef = f,ef
            if em > 0.5:
                m,em = Counts2Mag(3*ef,ef,zp)
                em = 99
        elif f <= 0 and ef > 0:
            m,em = Counts2Mag(3*ef,ef,zp)
            em = 99
            of,oef = 0., ef
        else:
            m = 99
            em = 99
            of = -99
            oef = -99
        #
        if numpy.isnan(m) or numpy.isnan(em):
            m = 99
            em = 99
        if numpy.isnan(f) or numpy.isnan(ef):
            of = -99
            oef = -99
        #
        mag.append(m)
        emag.append(em)
        oflux.append(of)
        oeflux.append(oef)
    #
    resdict = {"mags":numpy.array(mag),"emags":numpy.array(emag),"fluxes":numpy.array(oflux),"efluxes":numpy.array(oeflux)}
    return resdict



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.1.0
Author  : Stefano Covino
Date    : 22/08/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetApPhot

Remarks :

History : (25/02/2016) First version.
        : (22/08/2017) SRPFITS version.
"""


import SRPGW as GW
import SRPFITS.Photometry.DaoPhot as SPD
#import numpy
#from PythonPhot import aper


def DaoPhot(x,y,data,rds=(5,10,15),backgr=True):
    return SPD.DaoPhot(x,y,data,rds,backgr,GW.VSTgain,GW.VSTron)






""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 12/11/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : FindSubPathes

Remarks :

History : (12/11/2015) First version.
"""



def FindPathes (col1, col2):
    posset = set()
    for p1,p2 in zip(col1,col2):
        posset.add((p1,p2))
    #
    posstr = list(posset)
    return posstr


""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.2.2
Author  : Stefano Covino
Date    : 21/12/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : Score

Remarks :

History : (06/03/2016) First version.
        : (18/03/2016) Lower weight to magnitude difference.
        : (24/12/2016) Chi2 in score.
        : (18/01/2017) Higher score for nearby galaxies.
        : (21/12/2017) More moderate score for nearby galaxies.
"""

import numpy


def NeiFct (neigh):
    fn = 0.20*neigh+0.0
    return numpy.where(neigh > 5, 1.0, fn)

def DNeiFct (dneigh):
    fd  = 0.2*(dneigh-2.5)+1.0
    return numpy.where(dneigh > 2.5, 1.0, numpy.where(fd < 0.0, 0.0, fd))

def neiWei (neigh,dneigh):
    return numpy.where((dneigh>2.5) | (neigh>5), 1.0, (NeiFct(neigh)+DNeiFct(dneigh))/2.)

def magWei(magave):
    return (30.-magave)/30.




def Score(magave,varmag,neigh,dneigh,closegal=-99,chi2=-99):
    #
    sco = varmag * neiWei(neigh, dneigh) + magWei(magave)
    #
    sco = numpy.where(closegal != -99, sco + 0.75, sco)
    #
    sco = numpy.where(chi2 != -99, sco + chi2/100., sco)
    #
    return 100*sco



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/10/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)

Remarks :

History : (29/10/2015) First version.
"""


from astropy.coordinates import SkyCoord
from astropy import units as u
from astroquery.simbad import Simbad
import warnings



def GetSimbad(rad,decd,rds):
    crd = SkyCoord(ra=rad*u.degree, dec=decd*u.degree)
    #
    warnings.filterwarnings('ignore', category=UserWarning, append=True)
    try:
        restab = Simbad.query_region(crd, radius=rds*u.arcsec)
    except:
        restab = None
    warnings.resetwarnings()
    if restab is None:
        obj = 'No'
    else:
        #if restab['MAIN_ID'].size == 1:
        #    obj = restab['MAIN_ID'][0].decode().strip().replace(' ','_')
        #else:
        #    obj = restab['MAIN_ID'][1].decode().strip().replace(' ','_')
        obj = restab[0]['MAIN_ID'].decode().strip().replace(' ','_')
    #
    return obj




from ez_setup import use_setuptools
use_setuptools()

from setuptools import setup, find_packages 
import glob, os.path, sys


# Path check
def which(command):
    percorso = os.getenv("PATH")
    directories = percorso.split(os.pathsep)
    for path_dir in directories:
        real_dir = os.path.expanduser(path_dir)
        try:
            lista_dir = os.listdir(real_dir)
        except OSError:
            lista_dir = []
        if os.path.exists(real_dir) and command in lista_dir:
            return os.path.join(real_dir, command)
    return None


# Look for script files
lscr = glob.glob(os.path.join('Scripts', 'SRP*'))
lscrex = []
for i in lscr:
    if os.path.splitext(i)[1] == '':
        lscrex.append(i)



import SRPGW as GW



setup(
    name='SRPAstro.GW',
    version=GW.__version__,
    description='Tools for handling GW EM transient search under SRP',
    packages = find_packages('.'),
    include_package_data = False,
    long_description='Set of tools to handle GW EM transient search.',
    author='Stefano Covino', 
    author_email='stefano.covino@brera.inaf.it', 
    url='https://pypi.python.org/pypi/SRPAstro.GW',
    install_requires=['SRPAstro >= 4.2.13','PyPrind','mysql-connector-python','sep >= 0.5', 'photutils', 'astropy >= 1.1.1', 'aplpy', 'pyds9', 'SRPAstro.FITS >= 2.4.3',
        'astlib', 'feets'],
    scripts=lscrex,
    zip_safe = False,
    package_data={'SRPGW':['Data/*',]},
    classifiers=[ 
        'Development Status :: 5 - Production/Stable', 
        'Environment :: Console', 
        'Intended Audience :: Science/Research', 
        'License :: Freely Distributable', 
        'Operating System :: MacOS :: MacOS X', 
        'Operating System :: POSIX', 
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Scientific/Engineering :: Astronomy', 
        ], 
    ) 




""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.2.0
Author  : Stefano Covino
Date    : 30/08/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)

Remarks :

History : (02/11/2015) First version.
        : (27/01/2016) Minor correction.
        : (10/02/2016) Simpler and more rational version.
        : (30/08/2016) More robust against problems.
"""


import numpy
from astropy.io.fits import getdata
from astropy.wcs import WCS
from SRPGW.GetHeadVal import GetHeadVal



def GetPixVal(rad,decd,fname):
    dat = getdata(fname)
    w = WCS(fname)
    #
    nx1 = GetHeadVal('NAXIS1',fname)
    nx2 = GetHeadVal('NAXIS2',fname)
    #
    x, y = w.all_world2pix(rad,decd,0)
    #
    a = numpy.rint(x)-1
    b = numpy.rint(y)-1
    aa = a.astype(numpy.int)
    bb = b.astype(numpy.int)
    #
    inside = (aa >= 0) & (aa <= nx1) & (bb >= 0) & (bb <= nx2)
    #
    val = numpy.where(inside,dat[bb,aa],0.)
    #
    return val


""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.1.1
Author  : Stefano Covino
Date    : 24/02/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)

Remarks :

History : (03/04/2016) First version.
		: (26/05/2016) Working for arrays in input.
        : (24/02/2017) Better management of demanding WCS.
"""



from astropy.wcs import WCS
import astropy.wcs.wcs
import numpy




def IsCoordIn(rad,decd,fname):
    w = WCS(fname)
    #
    try:
        x, y = w.all_world2pix(rad,decd,0)
    except astropy.wcs.wcs.NoConvergence:
        x, y = w.wcs_world2pix(rad,decd,0)
    #
    X = numpy.array(x)
    Y = numpy.array(y)
    #
    return numpy.where((X >= 0) & (X <= w._naxis1) & (Y >= 0) & (Y <= w._naxis2), True, False)
